package ru.nsu.ccfit.boldyrev.minesweeper;

import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.FileReader;
import java.util.ArrayList;

import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

public abstract class Controller {
	public final String recordsTableFilename = "minesweeper_st0ke_records.txt";
	
	protected View view;
	protected Field field = null;
	protected ArrayList<Record> recordsTable = new ArrayList<Record>();

	Controller() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e1) {
			e1.printStackTrace();
			JOptionPane.showMessageDialog(null, e1.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (InstantiationException e1) {
			e1.printStackTrace();
			JOptionPane.showMessageDialog(null, e1.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (IllegalAccessException e1) {
			e1.printStackTrace();
			JOptionPane.showMessageDialog(null, e1.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (UnsupportedLookAndFeelException e1) {
			e1.printStackTrace();
			JOptionPane.showMessageDialog(null, e1.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
		}

		try {
			FileReader fr = new FileReader(System.getProperty("user.home") + "/" + recordsTableFilename);
			BufferedReader br = new BufferedReader(fr);

			String line;
			while ((line = br.readLine()) != null) {
				if (line.isEmpty()) {
					continue;
				}
				String[] args = line.split(":");
				if (args.length != 5) {
					throw new IOException("Bad file format");
				}

				recordsTable.add(new Record(Integer.valueOf(args[0]), Integer
						.valueOf(args[1]), Integer.valueOf(args[2]), args[3],
						Integer.valueOf(args[4])));
			}

			fr.close();
		} catch (IOException e) {
			/*
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
					*/
		}
	}

	public abstract boolean generate(int width, int height, int numberOfMines);

	public abstract OpenResult open(int x, int y, OpenType openType);

	public void updateRecordsTable(int width, int height, int numberOfMines,
			int time) {
		Record record = null;
		
		for (Record r : recordsTable) {
			if (r.getWidth() == width && r.getHeight() == height
					&& r.getNumberOfMines() == numberOfMines) {
				if (r.getTime() > time) {
					record = r;
					break;
				} else {
					return;
				}
			}
		}
		String name = view.getPlayerName();
		
		if (record == null) {
			recordsTable.add(new Record(width, height, numberOfMines, name,
					time));
		} else {
			record.change(name, time);
		}

		try {
			FileWriter fw = new FileWriter(System.getProperty("user.home") + "/" + recordsTableFilename);
			for (Record r : recordsTable) {
				fw.write(r.getWidth() + ":" + r.getHeight() + ":"
						+ r.getNumberOfMines() + ":" + r.getName() + ":"
						+ r.getTime() + "\n");
			}
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public ArrayList<Record> getRecordsTable() {
		return recordsTable;
	}

	public Field getField() {
		return field;
	}

	public class Record {
		private int width, height, numberOfMines;
		private String name;
		private int time;

		Record(int theWidth, int theHeight, int theNumberOfMines,
				String theName, int theTime) {
			name = theName;
			width = theWidth;
			height = theHeight;
			numberOfMines = theNumberOfMines;
			time = theTime;
		}

		public void change(String theName, int theTime) {
			name = theName;
			time = theTime;

			name.replaceAll(":", "");
		}

		public int getWidth() {
			return width;
		}

		public int getHeight() {
			return height;
		}

		public int getNumberOfMines() {
			return numberOfMines;
		}

		public String getName() {
			return name;
		}

		public int getTime() {
			return time;
		}
	}

	public enum OpenResult {
		OR_OK, OR_BOMB, OR_FINISH, OR_BADARGS, OR_ALREADYOPENED, OR_FLAG, OR_RECORDS
	}

	public enum OpenType {
		OT_SINGLE, OT_MULTI, OT_FLAG, OT_UNDEFINED
	}
	
	public View getView() {
		return view;		
	}
	public void setView(View theView) {
		view = theView;
	}
	public abstract int getTime();
	public abstract int getNumberOfCheckedMines();
	public abstract void stopTimer();
}
