package ru.nsu.ccfit.boldyrev.minesweeper;

import java.util.ArrayList;

public class Field {
	private ArrayList<CellType> field = null;
	private int width;
	private int height;
	private int numberOfMines;

	public enum CellType {
		CT_CLOSED, CT_OPENED, CT_BOMB, CT_FLAG, CT_FLAGGEDBOMB, CT_NOTACELL
	}

	public Field(int theWidth, int theHeight, int theNumberOfMines) {
		width = theWidth;
		height = theHeight;
		numberOfMines = theNumberOfMines;

		field = new ArrayList<CellType>();
		for (int i = 0; i < width * height; i++) {
			field.add(CellType.CT_CLOSED);
		}
	}

	public void change(int theWidth, int theHeight, int theNumberOfMines) {
		width = theWidth;
		height = theHeight;
		numberOfMines = theNumberOfMines;

		field.clear();
		for (int i = 0; i < width * height; i++) {
			field.add(CellType.CT_CLOSED);
		}
	}

	public CellType getCell(int offset) {
		if (offset < 0 || offset >= (width * height)) {
			return CellType.CT_NOTACELL;
		}
		return field.get(offset);
	}

	public CellType getCell(int x, int y) {
		if (x < 0 || y < 0 || x >= width || y >= height) {
			return CellType.CT_NOTACELL;
		}

		return field.get(x + y * width);
	}

	public void setCell(int offset, CellType cellType) {
		if (offset >= 0 && offset < (width * height)) {
			field.set(offset, cellType);
		}
	}

	public void setCell(int x, int y, CellType cellType) {
		if (x >= 0 && y >= 0 && x < width && y < height) {
			field.set(x + y * width, cellType);
		}
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}

	public int getNumberOfMines() {
		return numberOfMines;
	}
}
