package ru.nsu.ccfit.boldyrev.minesweeper;

import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

import ru.nsu.ccfit.boldyrev.minesweeper.Field.CellType;

public class SimpleController extends Controller {
	private Random random = new Random();

	private int numberOfOpenedCells = 0;
	private int numberOfCheckedMines = 0;

	private int time;
	private Timer timer = null;

	public SimpleController() {
	}

	private class TimeTimerTask extends TimerTask {
		@Override
		public void run() {
			time++;
			view.updateTime(time);
		}
	}

	@Override
	public boolean generate(int width, int height, int numberOfMines) {
		if (width < 1 || height < 1 || width > 30 || height > 24
				|| numberOfMines < 0 || numberOfMines > (width * height - 1)) {
			return false;
		}

		if (field != null) {
			field.change(width, height, numberOfMines);
		} else {
			field = new Field(width, height, numberOfMines);
		}

		for (int i = 0; i < width * height; i++) {
			field.setCell(i, Field.CellType.CT_CLOSED);
		}

		numberOfCheckedMines = 0;
		numberOfOpenedCells = 0;

		return true;
	}

	private void generateMines() {
		time = -1;
		if (timer != null) {
			timer.cancel();
		}
		timer = new Timer();
		timer.schedule(new TimeTimerTask(), 0, 1000);
		
		int numberOfSetted = 0;
		while (numberOfSetted != field.getNumberOfMines()) {
			for (int i = 0; i < field.getWidth() * field.getHeight(); i++) {
				if (field.getCell(i) == Field.CellType.CT_BOMB
						|| field.getCell(i) == Field.CellType.CT_OPENED) {
					continue;
				}
				if (random.nextInt(100) < 5) {
					if (field.getCell(i) == Field.CellType.CT_FLAG) {
						field.setCell(i, Field.CellType.CT_FLAGGEDBOMB);
					} else {
						field.setCell(i, Field.CellType.CT_BOMB);
					}
					numberOfSetted++;
					if (numberOfSetted == field.getNumberOfMines()) {
						break;
					}
				}
			}
		}
	}

	@Override
	public OpenResult open(int x, int y, OpenType openType) {
		if (field.getCell(x, y) == Field.CellType.CT_NOTACELL) {
			return OpenResult.OR_BADARGS;
		} else if (field.getCell(x, y) == Field.CellType.CT_BOMB
				&& openType != OpenType.OT_FLAG) {
			timer.cancel();
			timer = null;
			return OpenResult.OR_BOMB;
		} else if (field.getCell(x, y) == Field.CellType.CT_OPENED) {
			if (openType == OpenType.OT_MULTI) {
				int numberOfMines = 0, numberOfFlags = 0;
				for (int dx = -1; dx < 2; dx++) {
					for (int dy = -1; dy < 2; dy++) {
						if (dx == 0 && dy == 0) {
							continue;
						}
						if (field.getCell(x + dx, y + dy) == Field.CellType.CT_FLAG) {
							numberOfFlags++;
						} else if (field.getCell(x + dx, y + dy) == Field.CellType.CT_FLAGGEDBOMB) {
							numberOfMines++;
							numberOfFlags++;
						} else if (field.getCell(x + dx, y + dy) == Field.CellType.CT_BOMB) {
							numberOfMines++;
						}
					}
				}
				if (numberOfFlags == numberOfMines) {
					if (numberOfMines == 0) {
						return OpenResult.OR_BADARGS;
					}
					for (int dx = -1; dx < 2; dx++) {
						for (int dy = -1; dy < 2; dy++) {
							if (dx == 0 && dy == 0) {
								continue;
							}
							OpenResult res = open(x + dx, y + dy,
									OpenType.OT_SINGLE);
							if (res == OpenResult.OR_FINISH
									|| res == OpenResult.OR_BOMB) {
								return res;
							}
						}
					}
				}
			} else {
				return OpenResult.OR_BADARGS;
			}
		} else if (field.getCell(x, y) == Field.CellType.CT_FLAG
				|| field.getCell(x, y) == Field.CellType.CT_FLAGGEDBOMB) {
			if (openType == OpenType.OT_FLAG) {
				if (field.getCell(x, y) == Field.CellType.CT_FLAG) {
					field.setCell(x, y, Field.CellType.CT_CLOSED);
				} else {
					field.setCell(x, y, Field.CellType.CT_BOMB);
				}
				numberOfCheckedMines--;
			}
			return OpenResult.OR_FLAG;
		}
		if (openType == OpenType.OT_SINGLE) {
			field.setCell(x, y, CellType.CT_OPENED);
			numberOfOpenedCells++;

			if (numberOfOpenedCells == 1) {
				generateMines();
			}

			int numberOfMines = 0;
			for (int dx = -1; dx < 2; dx++) {
				for (int dy = -1; dy < 2; dy++) {
					if (dx == 0 && dy == 0) {
						continue;
					}
					if (field.getCell(x + dx, y + dy) == Field.CellType.CT_FLAGGEDBOMB) {
						numberOfMines++;
						break;
					} else if (field.getCell(x + dx, y + dy) == Field.CellType.CT_BOMB) {
						numberOfMines++;
						break;
					}
				}
			}
			if (numberOfMines == 0) {
				for (int dx = -1; dx < 2; dx++) {
					for (int dy = -1; dy < 2; dy++) {
						if (dx == 0 && dy == 0) {
							continue;
						}
						OpenResult res = open(x + dx, y + dy,
								OpenType.OT_SINGLE);
						if (res == OpenResult.OR_FINISH
								|| res == OpenResult.OR_BOMB) {
							return res;
						}
					}
				}
			}
		} else if (openType == OpenType.OT_MULTI) {
			return OpenResult.OR_BADARGS;
		} else if (openType == OpenType.OT_FLAG) {
			numberOfCheckedMines++;
			if (field.getCell(x, y) == Field.CellType.CT_BOMB) {
				field.setCell(x, y, CellType.CT_FLAGGEDBOMB);
			} else {
				field.setCell(x, y, CellType.CT_FLAG);
			}
		} else {
			return OpenResult.OR_ALREADYOPENED;
		}

		if (numberOfOpenedCells == field.getWidth() * field.getHeight()
				- field.getNumberOfMines()) {
			updateRecordsTable(field.getWidth(), field.getHeight(), field
					.getNumberOfMines(), time);
			timer.cancel();
			timer = null;
			return OpenResult.OR_FINISH;
		}
		return OpenResult.OR_OK;
	}

	public int getNumberOfOpenedCells() {
		return numberOfOpenedCells;
	}
	
	public int getNumberOfCheckedMines() {
		return numberOfCheckedMines;
	}

	public int getTime() {
		return time;
	}

	@Override
	public void stopTimer() {
		time = 0;
		if (timer != null) {
			timer.cancel();
			timer = null;
		}
		view.updateTime(0);
	}
}
