package ru.nsu.ccfit.boldyrev.minesweeper.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import ru.nsu.ccfit.boldyrev.minesweeper.Controller;
import ru.nsu.ccfit.boldyrev.minesweeper.Field;
import ru.nsu.ccfit.boldyrev.minesweeper.SimpleController;
import ru.nsu.ccfit.boldyrev.minesweeper.View;

public class SwingView extends JFrame implements View {
	private Controller controller = null;

	private static final long serialVersionUID = 1L;
	private JMenuItem jMenuItem0;
	private JMenuItem jMenuItem1;
	private JSeparator jSeparator0;
	private JMenuItem jMenuItem3;
	private JMenu jMenu0;
	private JMenuItem jMenuItem2;
	private JMenu jMenu1;
	private JMenuBar jMenuBar0;
	private JLabel jLabelBombs;
	private JLabel jLabelTime;
	private JPanel jPanelField;
	private JButton jButtonNewGame;
	private JButton[] cells;

	private Icon iconMine, iconFlag;

	private boolean bombVisibled = false;

	private int lastWidth = 9, lastHeight = 9, lastNumberOfMines = 10;

	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";

	public SwingView() {
		initComponents();
	}

	private void initComponents() {
		setTitle("Minesweeper");
		setLocationByPlatform(true);
		setResizable(true);
		setLayout(new GridBagLayout());
		setJMenuBar(getJMenuBar0());
		setSize(500, 400);

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.NONE;
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.NORTH;
		c.gridheight = 1;
		c.gridwidth = 1;
		c.weightx = 0.25;
		add(getJLabelBombs(), c);

		c.gridx = 1;
		c.gridy = 0;
		add(getJLabelTime(), c);

		c.gridx = 2;
		c.gridy = 0;
		c.weightx = 0.5;
		add(getJButtonNewGame(), c);

		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.gridy = 1;
		c.gridheight = GridBagConstraints.REMAINDER;
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1.0;
		add(getJPanelField(), c);

		ClassLoader cl = this.getClass().getClassLoader();
		iconFlag = new ImageIcon(
				cl
						.getResource("ru/nsu/ccfit/boldyrev/minesweeper/resources/flag.png"));
		iconMine = new ImageIcon(
				cl
						.getResource("ru/nsu/ccfit/boldyrev/minesweeper/resources/mine.png"));
	}

	private JButton getJButtonNewGame() {
		if (jButtonNewGame == null) {
			jButtonNewGame = new JButton();
			jButtonNewGame.setText("New game");
			jButtonNewGame.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					controller.stopTimer();
					startNewGame(lastWidth, lastHeight, lastNumberOfMines);
				}
			});
		}
		return jButtonNewGame;
	}

	private JPanel getJPanelField() {
		if (jPanelField == null) {
			jPanelField = new JPanel();
		}
		return jPanelField;
	}

	private JLabel getJLabelBombs() {
		if (jLabelBombs == null) {
			jLabelBombs = new JLabel();
			jLabelBombs.setText("Bombs: 10");
		}
		return jLabelBombs;
	}

	private JLabel getJLabelTime() {
		if (jLabelTime == null) {
			jLabelTime = new JLabel();
			jLabelTime.setText("Time: 0");
		}
		return jLabelTime;
	}

	private JMenuBar getJMenuBar0() {
		if (jMenuBar0 == null) {
			jMenuBar0 = new JMenuBar();
			jMenuBar0.add(getJMenu0());
			jMenuBar0.add(getJMenu1());
		}
		return jMenuBar0;
	}

	private JMenu getJMenu1() {
		if (jMenu1 == null) {
			jMenu1 = new JMenu();
			jMenu1.setText("Help");
			jMenu1.setBorderPainted(true);
			jMenu1.setOpaque(false);
			jMenu1.setRolloverEnabled(false);
			jMenu1.add(getJMenuItem2());
		}
		return jMenu1;
	}

	private JMenuItem getJMenuItem2() {
		if (jMenuItem2 == null) {
			jMenuItem2 = new JMenuItem();
			jMenuItem2.setText("About");
			jMenuItem2.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent event) {
					jMenuItem2ActionActionPerformed(event);
				}
			});
		}
		return jMenuItem2;
	}

	private JMenu getJMenu0() {
		if (jMenu0 == null) {
			jMenu0 = new JMenu();
			jMenu0.setText("File");
			jMenu0.setBorderPainted(true);
			jMenu0.setOpaque(false);
			jMenu0.setRolloverEnabled(false);
			jMenu0.add(getJMenuItem0());
			jMenu0.add(getJMenuItem1());
			jMenu0.add(getJSeparator0());
			jMenu0.add(getJMenuItem3());
		}
		return jMenu0;
	}

	private JMenuItem getJMenuItem3() {
		if (jMenuItem3 == null) {
			jMenuItem3 = new JMenuItem();
			jMenuItem3.setText("Exit");
			jMenuItem3.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent event) {
					jMenuItem3ActionActionPerformed(event);
				}
			});
		}
		return jMenuItem3;
	}

	private JSeparator getJSeparator0() {
		if (jSeparator0 == null) {
			jSeparator0 = new JSeparator();
		}
		return jSeparator0;
	}

	private JMenuItem getJMenuItem1() {
		if (jMenuItem1 == null) {
			jMenuItem1 = new JMenuItem();
			jMenuItem1.setText("Records");
			jMenuItem1.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent event) {
					jMenuItem1ActionActionPerformed(event);
				}
			});
		}
		return jMenuItem1;
	}

	private JMenuItem getJMenuItem0() {
		if (jMenuItem0 == null) {
			jMenuItem0 = new JMenuItem();
			jMenuItem0.setText("New game");
			jMenuItem0.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent event) {
					jMenuItem0ActionActionPerformed(event);
				}
			});
		}
		return jMenuItem0;
	}

	private static void installLnF() {
		try {
			String lnfClassname = PREFERRED_LOOK_AND_FEEL;
			if (lnfClassname == null)
				lnfClassname = UIManager.getCrossPlatformLookAndFeelClassName();
			UIManager.setLookAndFeel(lnfClassname);
		} catch (Exception e) {
			System.err.println("Cannot install " + PREFERRED_LOOK_AND_FEEL
					+ " on this platform:" + e.getMessage());
		}
	}

	/**
	 * Main entry of the class. Note: This class is only created so that you can
	 * easily preview the result at runtime. It is not expected to be managed by
	 * the designer. You can modify it as you like.
	 */
	public static void main(String[] args) {
		installLnF();
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				SwingView frame = new SwingView();
				frame.setController(new SimpleController());
				frame.getController().setView(frame);
				frame.setDefaultCloseOperation(SwingView.EXIT_ON_CLOSE);
				frame.setTitle("SwingView");
				frame.getContentPane().setPreferredSize(frame.getSize());
				frame.setLocationRelativeTo(null);
				frame.startNewGame(9, 9, 10);
				frame.pack();
				frame.setVisible(true);
			}
		});
	}

	private void jMenuItem3ActionActionPerformed(ActionEvent event) {
		this.dispose();
	}

	private void jMenuItem2ActionActionPerformed(ActionEvent event) {
		String msg;
		msg = "Minesweeper\n";
		msg += "Developed by Alexander Boldyrev [..st0ke]\n";
		msg += "E-Mail: i@st0ke.ru";
		JOptionPane.showMessageDialog(this, msg, "About",
				JOptionPane.INFORMATION_MESSAGE);
	}

	@Override
	public String getPlayerName() {
		return JOptionPane.showInputDialog(this,
				"New record! Please, enter your nickname.");
	}

	public Controller getController() {
		return controller;
	}

	public void setController(Controller controller) {
		this.controller = controller;
	}

	private void updateField(Controller.OpenResult result) {
		if (result == Controller.OpenResult.OR_BOMB || 
				result == Controller.OpenResult.OR_FINISH) {
			bombVisibled = true;
		} 

		jLabelBombs.setText("Bombs: "
				+ String.valueOf(controller.getField().getNumberOfMines()
						- controller.getNumberOfCheckedMines()));

		for (int y = 0; y < controller.getField().getHeight(); y++) {
			for (int x = 0; x < controller.getField().getWidth(); x++) {
				int i = x + y * controller.getField().getWidth();
				switch (controller.getField().getCell(i)) {
				case CT_BOMB:
					if (bombVisibled) {
						cells[i].setIcon(iconMine);
						cells[i].setDisabledIcon(iconMine);
						cells[i].setEnabled(true);
					} else {
						cells[i].setEnabled(true);
						cells[i].setIcon(null);
						cells[i].setDisabledIcon(null);
					}
					break;
				case CT_FLAG:
				case CT_FLAGGEDBOMB:
					cells[i].setIcon(iconFlag);
					cells[i].setDisabledIcon(iconFlag);
					cells[i].setEnabled(true);
					break;
				case CT_OPENED:
					cells[i].setEnabled(false);
					cells[i].setIcon(null);
					cells[i].setDisabledIcon(null);

					int numberOfMines = 0;
					for (int dx = -1; dx < 2; dx++) {
						for (int dy = -1; dy < 2; dy++) {
							if (dx == 0 && dy == 0) {
								continue;
							}
							if (controller.getField().getCell(x + dx, y + dy) == Field.CellType.CT_FLAGGEDBOMB
									|| controller.getField().getCell(x + dx,
											y + dy) == Field.CellType.CT_BOMB) {
								numberOfMines++;
							}
						}
					}
					if (numberOfMines > 0) {
						cells[i].setText(String.valueOf(numberOfMines));
					}
					break;
				case CT_CLOSED:
					cells[i].setEnabled(true);
					cells[i].setIcon(null);
					cells[i].setDisabledIcon(null);
					break;
				}
			}
		}
	}

	private class CellListener implements MouseListener {
		private int x;
		private int y;
		private Controller.OpenType openType = Controller.OpenType.OT_UNDEFINED;

		public CellListener(int theX, int theY) {
			x = theX;
			y = theY;
		}

		@Override
		public void mouseClicked(MouseEvent arg0) {
		}

		@Override
		public void mouseEntered(MouseEvent arg0) {
			cells[x + y * controller.getField().getWidth()].setSelected(true);
		}

		@Override
		public void mouseExited(MouseEvent arg0) {
			cells[x + y * controller.getField().getWidth()].setSelected(false);
		}

		@Override
		public void mousePressed(MouseEvent arg0) {
			if (bombVisibled) {
				return;
			}
			if (javax.swing.SwingUtilities.isLeftMouseButton(arg0)) {
				if (openType == Controller.OpenType.OT_FLAG) {
					openType = Controller.OpenType.OT_MULTI;
				} else {
					openType = Controller.OpenType.OT_SINGLE;
				}
			} else if (javax.swing.SwingUtilities.isRightMouseButton(arg0)) {
				if (openType == Controller.OpenType.OT_SINGLE) {
					openType = Controller.OpenType.OT_MULTI;
				} else {
					openType = Controller.OpenType.OT_FLAG;
				}
			}

			if (openType == Controller.OpenType.OT_MULTI) {
				for (int dx = -1; dx < 2; dx++) {
					for (int dy = -1; dy < 2; dy++) {
						if (dx == 0 && dy == 0) {
							continue;
						}
						if (controller.getField().getCell(x + dx, y + dy) == Field.CellType.CT_CLOSED
								|| controller.getField()
										.getCell(x + dx, y + dy) == Field.CellType.CT_BOMB) {
							cells[x + dx + (y + dy)
									* controller.getField().getWidth()]
									.setEnabled(false);
						}
					}
				}
			}
		}

		@Override
		public void mouseReleased(MouseEvent arg0) {
			if (bombVisibled) {
				return;
			}
			if (openType == Controller.OpenType.OT_UNDEFINED) {
				return;
			}

			Controller.OpenResult res = controller.open(x, y, openType);
			openType = Controller.OpenType.OT_UNDEFINED;

			updateField(res);
		}
	}

	public void startNewGame(int width, int height, int numberOfMines) {
		lastWidth = width;
		lastHeight = height;
		lastNumberOfMines = numberOfMines;
		jPanelField.removeAll();
		jPanelField.setLayout(new GridLayout(height, width));

		bombVisibled = false;
		controller.generate(width, height, numberOfMines);

		if (controller != null && controller.getField() != null) {
			jLabelBombs.setText("Bombs: "
					+ String.valueOf(controller.getField().getNumberOfMines()
							- controller.getNumberOfCheckedMines()));
		}
		cells = new JButton[width * height];
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				cells[x + y * width] = new JButton();
				cells[x + y * width].setSize(32, 32);
				cells[x + y * width].setMinimumSize(new Dimension(32, 32));
				cells[x + y * width].setMaximumSize(new Dimension(32, 32));
				cells[x + y * width].setFont(new Font("Tahoma", 0, 10));
				cells[x + y * width].setIcon(null);
				cells[x + y * width].setDisabledIcon(null);

				cells[x + y * width].addMouseListener(new CellListener(x, y));
				jPanelField.add(cells[x + y * width]);
			}
		}
		this.pack();
	}

	private void jMenuItem0ActionActionPerformed(ActionEvent event) {
		controller.stopTimer();
		JDialog jd = new SwingNewGameDialog(this);
		jd.pack();
		jd.setVisible(true);
	}

	private void jMenuItem1ActionActionPerformed(ActionEvent event) {
		JDialog jd = new SwingRecordsDialog(this);
		jd.pack();
		jd.setVisible(true);
	}

	public void updateTime(int time) {
		jLabelTime.setText("Time: " + time);
	}
}
