package ru.nsu.ccfit.boldyrev.minesweeper.text;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;

import ru.nsu.ccfit.boldyrev.minesweeper.Controller;
import ru.nsu.ccfit.boldyrev.minesweeper.Field;
import ru.nsu.ccfit.boldyrev.minesweeper.SimpleController;
import ru.nsu.ccfit.boldyrev.minesweeper.View;
import ru.nsu.ccfit.boldyrev.minesweeper.Controller.Record;

public class ConsoleView implements View {
	private Controller controller = null;
	private boolean gameStarted = false;
	private BufferedReader in = null;

	public ConsoleView(Controller theController) {
		controller = theController;
		controller.setView(this);
	}

	private void printHelp() {
		System.out.println("exit - stop this program");
		System.out.println("help - print this help message");
		System.out.println("about - print about message");
		System.out.println("records - print about message");
		System.out.println("new [width] [height] [number of mines] - new game (0 < width < 31, 0 < height < 25)");
		System.out.println("new - equal new 9 9 10");
		System.out.println("open [x] [y] [single|multi|flag] - open cell or flag it");
	}

	private void printField() {
		Field field = controller.getField();
		for (int y = 0; y < field.getHeight(); y++) {
			System.out.print(String.format("%3d", y) + "|");
			for (int x = 0; x < field.getWidth(); x++) {
				switch (field.getCell(x, y)) {
				case CT_OPENED:
					int numberOfMines = 0;
					for (int dx = -1; dx < 2; dx++) {
						for (int dy = -1; dy < 2; dy++) {
							if (dx == 0 && dy == 0) {
								continue;
							}
							if (field.getCell(x + dx, y + dy) == Field.CellType.CT_FLAGGEDBOMB
									|| field.getCell(x + dx, y + dy) == Field.CellType.CT_BOMB) {
								numberOfMines++;
							}
						}
					}
					if (numberOfMines == 0) {
						System.out.print(" ");
					} else {
						System.out.print(numberOfMines);
					}
					break;
				case CT_CLOSED:
					System.out.print("*");
					break;
				case CT_FLAGGEDBOMB:
				case CT_FLAG:
					System.out.print("f");
					break;
				case CT_BOMB:
					if (gameStarted) {
						System.out.print("*");
					} else {
						System.out.print("b");
					}
					break;
				}
			}
			System.out.println("|");
		}
	}

	public void run() {
		InputStreamReader converter = new InputStreamReader(System.in);
		in = new BufferedReader(converter);

		try {
			while (true) {
				System.out.print("$> ");
				String line = in.readLine();
				if (line == null) {
					break;
				}
				String[] cmd = line.split(" ");

				if (cmd.length == 0) {
					continue;
				}

				if (cmd[0].equals("exit") && cmd.length == 1) {
					break;
				} else if (cmd[0].equals("help") && cmd.length == 1) {
					printHelp();
				} else if (cmd[0].equals("records") && cmd.length == 1) {
					System.out.println("Records table: ");
					ArrayList<Record> records = controller.getRecordsTable();
					for (Record r : records) {
						String str = r.getName();
						str += " (" + r.getWidth() + "x" + r.getHeight() + ", " + r.getNumberOfMines() + ")";
						str += " --- " + r.getTime() + " sec.";
						System.out.println(str);
					}
				} else if (cmd[0].equals("about") && cmd.length == 1) {
					System.out.println("Minesweeper");
					System.out.println("Developed by Alexander Boldyrev [..st0ke]");
					System.out.println("E-Mail: i@st0ke.ru");
				} else if (cmd[0].equals("new") && cmd.length == 4) {
					if (!controller.generate(Integer.valueOf(cmd[1]), Integer
							.valueOf(cmd[2]), Integer.valueOf(cmd[3]))) {
						System.out.println("Bad field parameters");
						continue;
					}
					gameStarted = true;
					printField();
				} else if (cmd[0].equals("new") && cmd.length == 1) {
					if (!controller.generate(9,9,10)) {
						System.out.println("Bad field parameters");
						continue;
					}
					gameStarted = true;
					printField();
				} else if (cmd[0].equals("open") && cmd.length == 4) {
					if (!gameStarted) {
						System.out.println("Not in game\n");
						continue;
					}
					Controller.OpenType openType;
					if (cmd[3].equals("single")) {
						openType = Controller.OpenType.OT_SINGLE;
					} else if (cmd[3].equals("multi")) {
						openType = Controller.OpenType.OT_MULTI;
					} else if (cmd[3].equals("flag")) {
						openType = Controller.OpenType.OT_FLAG;
					} else {
						System.out.println("Bad last agrument, set to single\n");
						printHelp();
						continue;
					}
					Controller.OpenResult res = controller
							.open(Integer.valueOf(cmd[1]), Integer
									.valueOf(cmd[2]), openType);
					if (res == Controller.OpenResult.OR_FINISH) {
						System.out.println("You win! (time: "
								+ controller.getTime() + " seconds)");
						gameStarted = false;
					} else if (res == Controller.OpenResult.OR_BOMB) {
						System.out.println("You lose");
						gameStarted = false;
					} else if (res == Controller.OpenResult.OR_BADARGS) {
						System.out.println("Bad position");
					} else if (res == Controller.OpenResult.OR_ALREADYOPENED) {
						System.out.println("Cell is already opened");
					} else {
						System.out.println("time: " + controller.getTime()
								+ " seconds");
					}
					printField();
				} else if (cmd[0].equals("about") && cmd.length == 1) {
				} else {
					System.out.println("bad command or command's usage\n");
					printHelp();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		System.out.println("Goodbye");

	}

	public static void main(String[] args) {
		ConsoleView view = new ConsoleView(new SimpleController());
		view.run();
	}

	@Override
	public String getPlayerName() {
		if (in == null)
			return "";

		System.out.print("New Record! Enter your nickname: ");
		try {
			return in.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return "";
	}
	
	public void updateTime(int time) {
	}
}
